// code by jph
// http://www.midi.org/techspecs/midimessages.php
// http://computermusicresource.com/Control.Change.html
// http://tweakheadz.com/midi-controllers/
package mid;

import javax.sound.midi.ShortMessage;
import javax.sound.midi.SysexMessage;

/** for more constants see {@link ShortMessage} */
public abstract class MidiConstants {
  public static final int TEXT = 0x01; // in Sibelius Track 0: composer/comment
  public static final int COPYRIGHT_NOTICE = 0x02;
  public static final int TRACK_NAME = 0x03; // in Sibelius Track 0: title of piece
  /** The name of an instrument in the current track */
  public static final int INSTRUMENT_NAME = 0x04;
  /** Lyrics; in any track (not just 0). */
  public static final int LYRICS = 0x05;
  /** The text of a marker; in any track (not just 0). */
  public static final int MARKER = 0x06;
  public static final int CUE_POINT = 0x07;
  public static final int PROGRAM_NAME = 0x08;
  public static final int DEVICE_NAME = 0x09;
  public static final int CHANNEL_PREFIX = 0x20; // FF 20 01 cc
  public static final int PORT = 0x21; // FF 21 01 pp
  /** Mandatory event must be the last event in each MTrk chunk, and that should be the only occurrence per track. */
  public static final int TRACK_END = 0x2f; //
  public static final int TEMPO = 0x51;
  /** SMPTE time to denote playback offset from the beginning */
  public static final int SMPTE_OFFSET = 0x54;
  /** Time signature, metronome clicks, and size of a beat in 32nd notes */
  public static final int TIME_SIGNATURE = 0x58;
  /** byte 0 is inverted tonic byte 1 is flag for major/minor */
  public static final int KEY_SIGNATURE = 0x59;
  public static final int SEQUENCER_SPECIFIC = 0x7F; //
  // ---
  // for CONTROL_CHANGE
  public static final int BANK_SELECT = 0x00;
  public static final int MODULATION_WHEEL = 0x01;
  public static final int BREATH_CONTROLLER = 0x02;
  public static final int DATA_ENTRY_MSB = 0x06;
  public static final int VOLUME = 0x07;
  public static final int PAN_POSITION = 0x0a;
  public static final int SUSTAIN_PEDAL = 0x40; // Damper Pedal on/off
  public static final int PORTAMENTO = 0x41;
  public static final int SOSTENUTO_PEDAL = 0x42;
  public static final int SOFT_PEDAL = 0x43;
  public static final int LEGATO_FOOTSWITCH = 0x44;
  /** "hold 2 pedal", "harmonic pedal" in Pianoteq see also <a href="http://www.harmonicpianopedal.com/">harmonicpianopedal.com</a> */
  public static final int HARMONIC_PEDAL = 0x45; // previously ALT_SUSTAIN_PEDAL
  /** default: Reverb Send Level - see MMA RP-023 formerly External Effects Depth */
  public static final int EFFECTS_1_DEPTH = 0x5b; // 91
  /** formerly Tremolo Depth */
  public static final int EFFECTS_2_DEPTH = 0x5c; // 92
  /** default: Chorus Send Level - see MMA RP-023 formerly Chorus Depth */
  public static final int EFFECTS_3_DEPTH = 0x5d; // 93
  /** formerly Celeste [Detune] Depth */
  public static final int EFFECTS_4_DEPTH = 0x5e; // 94
  /** formerly Phaser Depth */
  public static final int EFFECTS_5_DEPTH = 0x5f; // 95
  /** Non-Registered Parameter Number (NRPN) - LSB */
  public static final int NRPN_LSB = 0x62; // 98
  /** Non-Registered Parameter Number (NRPN) - MSB */
  public static final int NRPN_MSB = 0x63; // 99
  public static final int ALL_SOUND_OFF = 0x78; // 120
  public static final int ALL_CONTROLLERS_OFF = 0x79; // 121
  public static final int ALL_NOTES_OFF = 0x7b; // 123
  // ---
  public static final int mask_0xf0 = 0xf0;
  public static final int SYSTEM_EXCLUSIVE = SysexMessage.SYSTEM_EXCLUSIVE; // 0xf0;
  // ---
  public static final int PITCH_BEND_CENTER = 0x2000; // 8192
  public static final int MAX_DETUNE_VALUE = 0x3fff; // 16383
  // ---
  /** convention: middle C = C4
   * 
   * standard piano range 88 key: 21...60...108 extended piano range 97 key: 12...60...108 */
  public static final int MIDDLE_C4 = 60;
  /** default velocity for note off command */
  public static final int NOTEOFF_VELOCITY_DEFAULT = 127;
  // ---
  /** MidiSystem.getMidiFileTypes() == new int[] {0, 1} but format type 0 throws "MIDI file type is not supported" java supports MIDI format type 1 */
  public static final int SUPPORTED_FILETYPE = 1;
  // ---
  private static final int[] framerate = new int[] { 24, 25, 30, 30 };

  public static int getFrameRate(byte myByte) {
    return framerate[myByte >>> 5];
  }

  public static int crop(int myInt) {
    return Math.min(Math.max(0, myInt), 127);
  }
}
