// code by jph
package mid.rec;

import java.util.Arrays;
import java.util.HashSet;
import java.util.NavigableMap;
import java.util.Set;
import java.util.TreeMap;

/** receives midi messages for instance from the LPK25 keyboard. the range of note pitch values is 0 ... 120. audible/valid piano keys are 21 to 108 (88 keys),
 * or 12 to 108 (97 keys). */
public abstract class SustainableReceiver extends EventReceiver {
  final int offset;
  int[] velocity = new int[121]; // LPK25
  boolean sustain = false;
  Set<Integer> mySustain = new HashSet<>();
  Set<Integer> myDowners = new HashSet<>();

  /** @param offset middle C corresponds to pitch 60-offset */
  public SustainableReceiver(int offset) {
    this.offset = offset;
  }

  @Override
  public void noteOn(int key, int velocity, long myLong) {
    this.velocity[key] = velocity;
    key = key2pitch(key);
    mySustain.add(key);
    myDowners.add(key);
    keyDown(key, velocity, myLong);
  }

  @Override
  public void noteOff(int key, long myLong) {
    // for LPK25 the NOTE_OFF velocity in myByte[2] is always 127!
    key = key2pitch(key);
    if (!sustain) {
      velocity[key + offset] = 0;
      mySustain.remove(key);
      keysUp(new HashSet<Integer>(Arrays.asList(key)), myLong); // new Integer[] { key }
    }
    myDowners.remove(key);
  }

  @Override
  public void sustain(boolean sustain, int myInt, long myLong) {
    this.sustain = sustain;
    if (!sustain) {
      mySustain.removeAll(myDowners);
      if (!mySustain.isEmpty()) {
        for (int sus : mySustain)
          velocity[sus + offset] = 0;
        keysUp(mySustain, myLong);
        mySustain.clear();
      }
    }
  }

  private int key2pitch(int key) {
    return key - offset;
  }

  /** @param myLong micro seconds, for instance 88681000, corresponds to 88.681 seconds after starting
   * @param pitch
   * @param velocity */
  public abstract void keyDown(int pitch, int velocity, long myLong);

  /** @param myLong
   * @param mySet release of sustain might cause multiple notes to be lifted */
  public abstract void keysUp(Set<Integer> mySet, long myLong);

  public NavigableMap<Integer, Integer> getVelocityMap(int thresh) {
    NavigableMap<Integer, Integer> myNavigableMap = new TreeMap<>();
    // extended piano range 97 keys
    for (int c0 = 12; c0 <= 108; ++c0)
      if (thresh < velocity[c0])
        myNavigableMap.put(c0, velocity[c0]);
    return myNavigableMap;
  }

  public NavigableMap<Integer, Integer> getVelocityMap() {
    return getVelocityMap(0);
  }
}