// code by jph
package mid.rmi;

import java.io.ByteArrayOutputStream;

import javax.sound.midi.MidiMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.Sequence;

import mid.MidiConstants;
import mid.MidiOutput;

public class RemoteMidiOutputClient implements MidiOutput {
  private final RemoteMidiOutput myRemoteMidiOutput;

  public RemoteMidiOutputClient(RemoteMidiOutput myRemoteMidiOutput) {
    this.myRemoteMidiOutput = myRemoteMidiOutput;
  }

  @Override
  public void startSequence(Sequence mySequence) {
    ByteArrayOutputStream myByteArrayOutputStream = new ByteArrayOutputStream();
    try {
      MidiSystem.write(mySequence, MidiConstants.SUPPORTED_FILETYPE, myByteArrayOutputStream);
      myRemoteMidiOutput.startSequenceBytes(myByteArrayOutputStream.toByteArray());
    } catch (Exception myException) {
      myException.printStackTrace();
    }
  }

  @Override
  public boolean isRunning() {
    try {
      return myRemoteMidiOutput.isRunning();
    } catch (Exception myException) {
      throw new RuntimeException(myException.getMessage());
    }
  }

  @Override
  public long getTickPosition() {
    try {
      return myRemoteMidiOutput.getTickPosition();
    } catch (Exception myException) {
      throw new RuntimeException(myException.getMessage());
    }
  }

  @Override
  public void setTempoInBPM(float myFloat) {
    try {
      myRemoteMidiOutput.setTempoInBPM(myFloat);
    } catch (Exception myException) {
      throw new RuntimeException(myException.getMessage());
    }
  }

  @Override
  public void stopSequencers() {
    try {
      myRemoteMidiOutput.stopSequencers();
    } catch (Exception myException) {
      throw new RuntimeException(myException.getMessage());
    }
  }

  @Override
  public void send(MidiMessage myMidiMessage, long ticks) {
    try {
      myRemoteMidiOutput.sendMessageBytes(myMidiMessage.getMessage(), ticks);
    } catch (Exception myException) {
      throw new RuntimeException(myException.getMessage());
    }
  }
}
