// code by jph
package mid.rmi;

import java.io.ByteArrayInputStream;
import java.rmi.Naming;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import javax.sound.midi.MidiMessage;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.Sequence;

import mid.MidiOutputServer;

public class RemoteMidiOutputServer extends UnicastRemoteObject implements RemoteMidiOutput, AutoCloseable {
  public static final int OBJECT_SERVER_PORT = 9379;
  // ---
  public static final String prefix = RemoteMidiOutput.class.getSimpleName() + ".";
  public static final String host = "//localhost:" + OBJECT_SERVER_PORT + "/";
  // ---
  private final MidiOutputServer myMidiOutputServer;
  private final String bind;

  public RemoteMidiOutputServer(MidiOutputServer myMidiOutputServer) throws Exception {
    this.myMidiOutputServer = myMidiOutputServer;
    bind = host + prefix + myMidiOutputServer.getDeviceInfo();
    Naming.bind(bind, this);
  }

  @Override
  public void startSequenceBytes(byte[] myByte) throws RemoteException {
    try {
      Sequence mySequence = MidiSystem.getSequence(new ByteArrayInputStream(myByte));
      myMidiOutputServer.startSequence(mySequence);
    } catch (Exception myException) {
      myException.printStackTrace();
    }
  }

  @Override
  public boolean isRunning() throws RemoteException {
    return myMidiOutputServer.isRunning();
  }

  @Override
  public long getTickPosition() throws RemoteException {
    return myMidiOutputServer.getTickPosition();
  }

  @Override
  public void setTempoInBPM(float myFloat) throws RemoteException {
    myMidiOutputServer.setTempoInBPM(myFloat);
  }

  @Override
  public void stopSequencers() throws RemoteException {
    myMidiOutputServer.stopSequencers();
  }

  @Override
  public void sendMessageBytes(byte[] myByte, long ticks) throws RemoteException {
    try {
      MidiMessage myMidiMessage = new MidiMessage(myByte) {
        @Override
        public Object clone() {
          throw new RuntimeException("clone not implemented");
        }
      };
      myMidiOutputServer.send(myMidiMessage, ticks);
    } catch (Exception myException) {
      myException.printStackTrace();
    }
  }

  @Override
  public void close() throws Exception {
    Naming.unbind(bind);
    myMidiOutputServer.close();
  }
}
