// code by jph
package sys;

import java.awt.Component;
import java.awt.Dimension;
import java.io.File;
import java.util.Arrays;
import java.util.Collection;
import java.util.function.Consumer;

import javax.swing.JFileChooser;
import javax.swing.JMenuItem;
import javax.swing.filechooser.FileFilter;

import tpf.dat.loc.Local;

public abstract class CommonFileChooser {
  Manager myManager = Local.getInstance().getUserManager(CommonFileChooser.class);
  final Collection<String> myCollection;

  public CommonFileChooser(Collection<String> myCollection) {
    this.myCollection = myCollection;
  }

  private JFileChooser getJFileChooser() {
    JFileChooser myJFileChooser = new JFileChooser();
    myJFileChooser.setPreferredSize(new Dimension(800, 600));
    myJFileChooser.setCurrentDirectory(new File(myManager.getString("Directory", "")));
    myJFileChooser.setFileFilter(new FileFilter() {
      @Override
      public String getDescription() {
        return protected_getDescription();
      }

      @Override
      public boolean accept(File myFile) {
        return myFile.isDirectory() || myCollection.contains(new Filename(myFile).myExtension.toLowerCase());
      }
    });
    return myJFileChooser;
  }

  public final void add(JMenuItem myJMenuItem) {
    myJMenuItem.addActionListener(myActionEvent -> action(myJMenuItem));
  }

  public final void action(Component myComponent) {
    JFileChooser myJFileChooser = getJFileChooser();
    if (myJFileChooser.showOpenDialog(myComponent) == JFileChooser.APPROVE_OPTION)
      handle(myJFileChooser.getSelectedFile());
    File myFile = myJFileChooser.getCurrentDirectory();
    String myString = myFile.toString().replace('\\', '/');
    myManager.setProperty("Directory", myString);
    myManager.manifest();
  }

  abstract void handle(File myFile);

  abstract String protected_getDescription();

  public static CommonFileChooser of(Collection<String> myCollection, String myDescription, Consumer<File> myConsumer) {
    return new CommonFileChooser(myCollection) {
      @Override
      void handle(File myFile) {
        myConsumer.accept(myFile);
      }

      @Override
      String protected_getDescription() {
        return myDescription;
      }
    };
  }

  public static CommonFileChooser forMidi(Consumer<File> myConsumer) {
    return of(Arrays.asList(new String[] { "mid", "midi" }), "MIDI files", myConsumer);
  }

  public static CommonFileChooser forWav(Consumer<File> myConsumer) {
    return of(Arrays.asList(new String[] { "wav" }), "Wav files", myConsumer);
  }

  public static CommonFileChooser forMp3(Consumer<File> myConsumer) {
    return of(Arrays.asList(new String[] { "mp3" }), "MP3 files", myConsumer);
  }
}
