// code by jph
package sys;

import java.awt.Desktop;
import java.io.File;
import java.net.URI;

public class DesktopHelper {
  private static OperatingSystem operatingSystem = null;

  public static boolean isWindows() {
    return System.getProperty("os.name").startsWith("Windows");
  }

  public static boolean isLinux() {
    return System.getProperty("os.name").equals("Linux");
  }

  private static OperatingSystem determineOperatingSystem() {
    if (isWindows())
      return OperatingSystem.Windows;
    if (isLinux())
      return OperatingSystem.Linux;
    return OperatingSystem.Unknown;
  }

  public static OperatingSystem getOperatingSystem() {
    if (operatingSystem == null)
      operatingSystem = determineOperatingSystem();
    return operatingSystem;
  }

  /** @param myFile intended to open in explorer window while selected */
  public static void navigateTo(final File myFile) {
    if (myFile.exists()) {
      boolean opened = false;
      if (DesktopHelper.isWindows())
        try {
          new ProcessBuilder("explorer.exe", "/select,\"" + myFile + "\"").start();
          opened = true;
        } catch (Exception myException) {
          myException.printStackTrace();
        }
      if (!opened)
        open(myFile.isDirectory() ? myFile : myFile.getParentFile());
    } else
      new RuntimeException(myFile + " does not exist").printStackTrace();
  }

  /** can be used to open folders in explorer (without selecting any specific file in the folder)
   * 
   * @param myFile */
  public static void open(final File myFile) {
    if (myFile.exists())
      try {
        Desktop.getDesktop().open(myFile);
      } catch (Exception myException) {
        myException.printStackTrace();
      }
  }

  /** @param myString is address in browser */
  public static boolean browse(String myString) {
    try {
      Desktop.getDesktop().browse(new URI(myString));
      return true;
    } catch (Exception myException) {
      myException.printStackTrace();
    }
    return false;
  }
}
