// code by jph adapted from Jean-Francois Briere
// http://www.velocityreviews.com/forums/t137115-preventing-multiple-instance-standalone-desktop-gui-applications.html
package sys;

import java.io.File;
import java.io.RandomAccessFile;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;

import javax.swing.JOptionPane;

public class FileBlock {
  String myString;
  FileChannel myFileChannel;
  FileLock myFileLock;
  File myFile;

  private FileBlock(String myString) {
    this.myString = myString;
//    System.out.println("block " + myString);
  }

  public FileBlock(Class<?> myClass) {
    this(myClass.getCanonicalName());
  }

  public boolean isActive() {
    try {
      myFile = new File(System.getProperty("user.home"), myString + ".lock");
      myFile.deleteOnExit();
      myFileChannel = new RandomAccessFile(myFile, "rw").getChannel();
      myFileLock = myFileChannel.tryLock();
      if (myFileLock == null) {
        myFileChannel.close();
        return true;
      }
      Runtime.getRuntime().addShutdownHook(new Thread() {
        @Override
        public void run() {
          release();
        }
      });
      return false;
    } catch (Exception myException) {
      release();
    }
    return true;
  }

  void release() {
    try {
      myFileLock.release();
    } catch (Exception myException) {
      // System.out.println("FileLock (ignore!): " + myException);
    }
    try {
      myFileChannel.close();
    } catch (Exception myException) {
      System.out.println("FileChannel (ignore!): " + myException);
    }
  }

  public boolean defaultMessage() {
    boolean myBoolean = isActive();
    if (myBoolean)
      showMessage();
    return myBoolean;
  }

  void showMessage() {
    JOptionPane.showMessageDialog(null, myString + " is already running.", "Execution blocked", JOptionPane.ERROR_MESSAGE);
  }
}