// code by jph
package sys;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.Arrays;

public class FileUtils {
  public static boolean isIdentical(File myFile, File myDest) {
    return Arrays.equals(BinaryFile.read(myDest), BinaryFile.read(myFile));
  }

  public static boolean copyFileIfDifferent(File myFile, File myDest) throws IOException {
    if (myDest.exists() && isIdentical(myFile, myDest))
      return false;
    // ---
    // code modified from StackOverflow
    if (!myDest.exists())
      myDest.createNewFile();
    // ---
    FileInputStream myFileInputStream = null;
    FileChannel myFileChannel = null;
    try {
      myFileInputStream = new FileInputStream(myFile);
      myFileChannel = myFileInputStream.getChannel();
      // ---
      FileOutputStream myFileOutputStream = null;
      try {
        myFileOutputStream = new FileOutputStream(myDest);
        myFileOutputStream.getChannel().transferFrom(myFileChannel, 0, myFileChannel.size());
      } finally {
        if (myFileOutputStream != null)
          myFileOutputStream.close();
      }
    } finally {
      if (myFileChannel != null)
        myFileChannel.close();
      if (myFileInputStream != null)
        myFileInputStream.close();
    }
    return true;
  }

  public static void rigorousTransfer(File myFile, File myDst, int max_depth, int max_count) {
    if (!myFile.getName().equalsIgnoreCase(myDst.getName()))
      throw new RuntimeException();
    if (!myFile.exists())
      throw new RuntimeException("source " + myFile + " does not exist");
    // ---
    new FileDelete(myDst, max_depth, max_count);
    copyRecursively(myFile, myDst.getParentFile());
  }

  private static void copyRecursively(File myFile, File myDst) {
    if (myFile.isDirectory()) {
      File mySubDst = new File(myDst, myFile.getName());
      mySubDst.mkdirs(); // act of creation
      for (File myEntry : myFile.listFiles())
        copyRecursively(myEntry, mySubDst); // proceed on level up
    } else
      BinaryFile.write(new File(myDst, myFile.getName()), BinaryFile.read(myFile)); // act of creation
  }
}
