// code by jph
package sys;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GraphicsConfiguration;
import java.awt.GraphicsDevice;
import java.awt.GraphicsEnvironment;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.Toolkit;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;
import java.util.Random;
import java.util.Set;
import java.util.SortedSet;
import java.util.StringTokenizer;
import java.util.TreeSet;

public class Manager {
  /** Instances of java.util.Random are threadsafe. However, the concurrent use of the same java.util.Random instance across threads may encounter contention
   * and consequent poor performance. Consider instead using ThreadLocalRandom in multithreaded designs. */
  public static final Random random = new Random();

  public static Insets getScreenInsets(int myInt1, int myInt2) {
    Insets myInsets = null;
    try {
      GraphicsEnvironment myGraphicsEnvironment = GraphicsEnvironment.getLocalGraphicsEnvironment();
      GraphicsDevice[] myGraphicsDevice = myGraphicsEnvironment.getScreenDevices();
      GraphicsConfiguration[] myGraphicsConfiguration = myGraphicsDevice[myInt1].getConfigurations();
      myInsets = Toolkit.getDefaultToolkit().getScreenInsets(myGraphicsConfiguration[myInt2]);
    } catch (Exception myException) {
      myException.printStackTrace();
    }
    return myInsets;
  }

  public final List<String> myRemark = new LinkedList<>();
  public final List<String> myExecute = new LinkedList<>();
  private Properties myProperties = new Properties();
  private File myFile = null; // place to read and possibly update properties
  public boolean debug_display = true;

  public Manager(File myFile, Class<?> myClass) {
    List<String> myList = new ArrayList<>();
    while (myClass != Object.class) {
      myList.add(myClass.getName());
      myClass = myClass.getSuperclass();
    }
    Collections.reverse(myList);
    for (String myString : myList)
      load(new File(myFile, myString + ".properties"));
  }

  public Manager(Class<?> myClass) {
    this(new File("get", "properties"), myClass);
  }

  public Manager(File myFile) {
    load(myFile);
  }

  public Manager() {
  }

  static final Charset charset = Charset.forName("UTF-8");

  private void load(File myFile) {
    try {
      this.myFile = myFile;
      boolean myBoolean = myFile.createNewFile();
      if (myBoolean)
        System.out.println("created " + myFile);
      {
        FileInputStream myFileInputStream = new FileInputStream(myFile);
        BufferedReader myBufferedReader = new BufferedReader(new InputStreamReader(myFileInputStream, charset));
        myProperties.load(myBufferedReader);
        myBufferedReader.close();
      }
      // ---
      {
        FileInputStream myFileInputStream = new FileInputStream(myFile);
        BufferedReader myBufferedReader = new BufferedReader(new InputStreamReader(myFileInputStream, charset));
        String myString = null;
        while ((myString = myBufferedReader.readLine()) != null) {
          if (myString.startsWith("#"))
            myRemark.add(myString.substring(1).trim());
          if (myString.startsWith("!"))
            myExecute.add(myString.substring(1).trim());
        }
        myBufferedReader.close();
      }
    } catch (Exception myException) {
      myException.printStackTrace();
    }
  }

  public void manifest() {
    if (myFile != null)
      try {
        OutputStream myOutputStream = new FileOutputStream(myFile);
        OutputStreamWriter myOutputStreamWriter = new OutputStreamWriter(myOutputStream, charset);
        Writer myWriter = new BufferedWriter(myOutputStreamWriter);
        if (!myRemark.isEmpty()) {
          for (String myString : myRemark)
            myWriter.write("# " + myString + '\n');
          myWriter.write('\n');
        }
        SortedSet<String> mySortedSet = new TreeSet<>(myProperties.stringPropertyNames());
        if (!mySortedSet.isEmpty()) {
          for (String myString : mySortedSet)
            myWriter.write(myString.trim() + "=" + myProperties.getProperty(myString).trim() + '\n');
          myWriter.write('\n');
        }
        if (!myExecute.isEmpty()) {
          for (String myString : myExecute)
            myWriter.write("! " + myString + '\n');
          myWriter.write('\n');
        }
        myWriter.close(); // Closes the stream, flushing it first.
      } catch (Exception myException) {
        myException.printStackTrace();
      }
  }

  /** @return The returned set is not backed by the Properties object. Changes to this Properties are not reflected in the set, or vice versa. */
  public Set<String> keySet() {
    return myProperties.stringPropertyNames();
  }

  public Set<Entry<String, String>> entrySet() { // this could be a stream, but would have to case Object -> String
    Map<String, String> myMap = new HashMap<>();
    myProperties.entrySet().stream() //
        .forEach(myEntry -> myMap.put((String) myEntry.getKey(), (String) myEntry.getValue()));
    return myMap.entrySet();
  }

  public void setProperty(String myKey, String myValue) {
    myProperties.setProperty(myKey, myValue);
  }

  public void setProperty(String myKey, int myInt) {
    myProperties.setProperty(myKey, "" + myInt);
  }

  public void remove(String myString) {
    myProperties.remove(myString);
  }

  public void clear() {
    myProperties.clear();
  }

  public boolean containsKey(String myString) {
    return myProperties.containsKey(myString);
  }

  public boolean getBoolean(String myString, boolean myBoolean) {
    try {
      return Boolean.parseBoolean(myProperties.getProperty(myString).trim());
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myBoolean;
  }

  public int getInteger(String myString, int myInteger) {
    try {
      return Integer.parseInt(myProperties.getProperty(myString).trim());
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myInteger;
  }

  public long getLong(String myString, long myLong) {
    try {
      return Long.parseLong(myProperties.getProperty(myString).trim());
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myLong;
  }

  public float getFloat(String myString, float myFloat) {
    try {
      return Float.parseFloat(myProperties.getProperty(myString).trim());
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myFloat;
  }

  public double getDouble(String myString, double myDouble) {
    try {
      return Double.parseDouble(myProperties.getProperty(myString).trim());
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myDouble;
  }

  public String getString(String myString, String myDefault) {
    try {
      if (myProperties.getProperty(myString) == null)
        throw new Exception("null");
      return myProperties.getProperty(myString).trim();
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myDefault.trim();
  }

  public List<String> getListString(String myString, String myDefault, String myDelim) {
    List<String> myList = new ArrayList<>();
    StringTokenizer myStringTokenizer = new StringTokenizer(getString(myString, myDefault).replace("[", "").replace("]", ""), myDelim);
    while (myStringTokenizer.hasMoreTokens())
      myList.add(myStringTokenizer.nextToken().trim());
    return myList;
  }

  public List<Integer> getListInteger(String myString, List<Integer> myDefault) {
    List<Integer> myList = new ArrayList<>();
    boolean myBoolean = containsKey(myString);
    if (myBoolean) {
      StringTokenizer myStringTokenizer = new StringTokenizer(getString(myString, "").replace('[', ' ').replace(']', ' ').replace(',', ' '));
      while (myStringTokenizer.hasMoreTokens()) {
        myString = myStringTokenizer.nextToken();
        try {
          myList.add(Integer.parseInt(myString));
        } catch (Exception myException) {
          error(myString, myException);
          myBoolean = false;
        }
      }
      if (myBoolean)
        return myList;
    }
    return myDefault;
  }

  public Insets getInsets(String myString, Insets myInsets) {
    List<Integer> myList = getListInteger(myString, new ArrayList<>());
    try {
      return new Insets(myList.get(0), myList.get(1), myList.get(2), myList.get(3));
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myInsets;
  }

  public Color getColor(String myString, Color myColor) {
    List<Integer> myList = getListInteger(myString, new ArrayList<>());
    try {
      return new Color(myList.get(0), myList.get(1), myList.get(2));
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myColor;
  }

  public Dimension getDimension(String myString, Dimension myDimension) {
    List<Integer> myList = getListInteger(myString, new ArrayList<>());
    try {
      return new Dimension(myList.get(0), myList.get(1));
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myDimension;
  }

  Rectangle getRectangle(String myString, Rectangle myRectangle) {
    List<Integer> myList = getListInteger(myString, new ArrayList<>());
    try {
      return new Rectangle(myList.get(0), myList.get(1), myList.get(2), myList.get(3));
    } catch (Exception myException) {
      error(myString, myException);
    }
    return myRectangle;
  }

  static Set<String> error = new HashSet<>();

  private void error(String myString, Exception myException) {
    if (debug_display && error.add(myString)) {
      System.out.println(myString + '@' + myFile + " error: " + myException);
      // myException.printStackTrace();
    }
  }
}
