// code by jph
package sys.col;

import java.awt.Color;

import sys.mat.Fraction;
import sys.mat.IntegerMath;
import sys.mat.Primes;

public class HuePalette extends Palette {
  /** for most applications {@link LumaPalette}.default13 is more suitable */
  public static Palette default13 = HuePalette.prime(13, new Fraction(2, 5)); // HuePalette.mod3(12);

  public static Palette even(int size) {
    double[] myDouble = new double[size];
    for (int index = 0; index < size; ++index)
      myDouble[index] = index / (double) size;
    return new HuePalette(myDouble);
  }

  public static Palette mod3(int size) {
    int packs = IntegerMath.ceilDiv(size, 3);
    size = packs * 3;
    double[] myDouble = new double[size];
    int count = 0;
    double offset = 0;
    for (int pack = 0; pack < packs; ++pack) {
      myDouble[count + 0] = offset + 0 / 3.;
      myDouble[count + 1] = offset + 1 / 3.;
      myDouble[count + 2] = offset + 2 / 3.;
      count += 3;
      offset += 1 / (double) size;
    }
    return new HuePalette(myDouble);
  }

  public static Palette prime(int size, Fraction myFraction) {
    size = Primes.greaterEq(size);
    double[] myDouble = new double[size];
    int count = 0;
    int incr = Math.max(1, myFraction.times(size).round());
    for (int index = 0; index < size; ++index) {
      myDouble[index] = count / (double) size;
      count += incr;
      count %= size;
    }
    return new HuePalette(myDouble);
  }

  // ---
  private final double[] myDouble;

  public HuePalette(double[] myDouble) {
    this.myDouble = myDouble;
  }

  @Override
  public Color getColor(int index, double saturation, double value, double alpha) {
    return new Hue(myDouble[index], saturation, value, alpha).rgba;
  }

  @Override
  public int size() {
    return myDouble.length;
  }
}
