// code by jph
package sys.dat;

import java.awt.Color;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;
import java.nio.charset.Charset;

/** HtmlUtf8 exports strings to html pages in utf-8 encoding. All logs of MissionControl are exported with HtmlUtf8. */
public abstract class HtmlUtf8 {
  static final Charset charset = Charset.forName("UTF-8");
  public final File myFile; // should be final ideally
  public boolean isClosed = false;

  protected HtmlUtf8(File myFile) {
    this.myFile = myFile;
  }

  public static HtmlUtf8 page(File myFile, boolean flushed) {
    if (myFile.exists())
      myFile.delete();
    String myString;
    myString = "<!DOCTYPE html>\n<html>\n<head>\n";
    myString += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n";
    myString += "<style type='text/css'>\n";
    myString += "  acronym { border-bottom: 0px; }\n";
    myString += "</style>\n";
    myString += "</head>\n<body>\n";
    HtmlUtf8 myHtmlUtf8 = flushed ? new FlushedHtmlUtf8(myFile) : new BufferedHtmlUtf8(myFile);
    myHtmlUtf8.append(myString);
    return myHtmlUtf8;
  }

  public static HtmlUtf8 pageHead(File myFile, boolean flushed, String myInsert) {
    if (myFile.exists())
      myFile.delete();
    String myString = "";
    myString += "<!DOCTYPE html>\n<html>\n<head>\n";
    myString += "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n";
    if (myInsert != null)
      myString += myInsert;
    myString += "<style type='text/css'>\n";
    myString += "  acronym { border-bottom: 0px; }\n";
    myString += "</style>\n";
    myString += "</head>\n";
    HtmlUtf8 myHtmlUtf8 = flushed ? new FlushedHtmlUtf8(myFile) : new BufferedHtmlUtf8(myFile);
    myHtmlUtf8.append(myString);
    return myHtmlUtf8;
  }

  public void append(Object myObject) {
    if (isClosed)
      throw new RuntimeException("html export to " + myFile + " already closed.");
    private_append(myObject);
  }

  public static String color(Color myColor) {
    return String.format("#%02x%02x%02x", myColor.getRed(), myColor.getGreen(), myColor.getBlue());
  }

  protected abstract void private_append(Object myObject);

  public void closeWithCounter() {
    append("\n<!-- Start of StatCounter Code -->\n");
    append("<script type=\"text/javascript\">\n");
    append("var sc_project=4740711;\n");
    append("var sc_invisible=1;\n");
    append("var sc_partition=54;\n");
    append("var sc_click_stat=1;\n");
    append("var sc_security=\"0f652dbf\";\n");
    append("</script>\n\n");
    append("<script type=\"text/javascript\"\n");
    append("src=\"http://www.statcounter.com/counter/counter.js\"></script><noscript><div\n");
    append("class=\"statcounter\"><a title=\"free web stats\"\n");
    append("href=\"http://www.statcounter.com/\" target=\"_blank\"><img\n");
    append("class=\"statcounter\"\n");
    append("src=\"http://c.statcounter.com/4740711/0/0f652dbf/1/\"\n");
    append("alt=\"free web stats\" ></a></div></noscript>\n");
    append("<!-- End of StatCounter Code -->\n");
    close();
  }

  public void close() {
    append("</body>\n</html>\n");
    isClosed = true;
  }

  /** @param myFile
   * @param myTitle
   * @param mySplit for instance: cols="300,*"
   * @param myFileStringL
   * @param myNameStringL
   * @param myFileStringR
   * @param myNameStringR */
  public static void index(File myFile, String myTitle, //
      String mySplit, // cols="300,*"
      String myFileStringL, String myNameStringL, //
      String myFileStringR, String myNameStringR) {
    try {
      StringBuilder myStringBuffer = new StringBuilder();
      myStringBuffer.append("<html>\n");
      if (myTitle != null && !myTitle.isEmpty())
        myStringBuffer.append("<head><title>" + myTitle + "</title></head>\n");
      myStringBuffer.append("<frameset " + mySplit + ">\n");
      myStringBuffer.append("<frame src=\"" + myFileStringL + "\" name=\"" + myNameStringL + "\">\n");
      myStringBuffer.append("<frame src=\"" + myFileStringR + "\" name=\"" + myNameStringR + "\">\n");
      myStringBuffer.append("</frameset>\n</html>\n");
      OutputStream myOutputStream = new FileOutputStream(myFile);
      myOutputStream.write(myStringBuffer.toString().getBytes(charset));
      myOutputStream.close();
    } catch (Exception myException) {
      myException.printStackTrace();
    }
  }
  // static void counter(Writer myWriter) throws Exception {
  // myWriter.write("<script type=\"text/javascript\">\n");
  // myWriter.write("var sc_project=5576678; \n");
  // myWriter.write("var sc_invisible=1; \n");
  // myWriter.write("var sc_partition=60; \n");
  // myWriter.write("var sc_click_stat=1; \n");
  // myWriter.write("var sc_security=\"92e0876d\";\n");
  // myWriter.write("</script>\n");
  // myWriter.write("<script type=\"text/javascript\" \n");
  // myWriter.write("src=\"http://www.statcounter.com/counter/counter.js\"></script><noscript><div \n");
  // myWriter.write("class=\"statcounter\"><a title=\"web counter\" \n");
  // myWriter.write("href=\"http://www.statcounter.com/free_hit_counter.html\" \n");
  // myWriter.write("target=\"_blank\"><img class=\"statcounter\" \n");
  // myWriter.write("src=\"http://c.statcounter.com/5576678/0/92e0876d/1/\" \n");
  // myWriter.write("alt=\"web counter\" ></a></div></noscript>\n");
  // }
}
