// code by jph
package sys.dat;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.nio.charset.Charset;
import java.util.Stack;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

// design is not final, but is adapted as needed
public abstract class XmlReader {
  /** guess UTF-8 ideally the charset should be taken from the first line, for instance: <?xml version="1.0" encoding='UTF-8' standalone='no' ?> */
  public static final Charset charset = Charset.forName("UTF-8");
  // ---
  private Stack<String> myStack = null;
  private int line;

  /** override to access comments
   * 
   * @param myGroup */
  protected void comment(String myGroup) {
  }

  /** @param myToken
   * @param myString is trimmed
   * @throws Exception */
  protected abstract void string(String myToken, String myString) throws Exception;

  protected abstract void pop(String myToken) throws Exception;

  /** myToken != peek()
   * 
   * @param myToken
   * @param myGroup
   * @throws Exception */
  protected abstract void pushpop(String myToken, String myGroup) throws Exception;

  protected abstract void push(String myToken, String myGroup) throws Exception;

  /** @param myInputStream is not closed inside parse function, but should to be closed outside to allow for processing of ZipEntry
   * @throws Exception */
  public final void parse(InputStream myInputStream) throws Exception {
    BufferedReader myBufferedReader = new BufferedReader(new InputStreamReader(myInputStream, charset));
    myStack = new Stack<>();
    line = 0;
    Pattern myPattern = Pattern.compile("<.+?>");
    StringBuffer myLineBuffer = new StringBuffer();
    String myLine;
    try {
      while ((myLine = myBufferedReader.readLine()) != null) {
        myLineBuffer.append(myLine);
        // String myString = myStringBuilder.toString();
        Matcher myMatcher = myPattern.matcher(myLineBuffer);
        while (myMatcher.find()) {
          StringBuffer myStringBuffer = new StringBuffer();
          myMatcher.appendReplacement(myStringBuffer, "");
          {
            String myContent = myStringBuffer.toString().trim();
            if (!myContent.isEmpty())
              if (myStack.isEmpty()) {
                System.out.println("[" + line + "] " + myLineBuffer);
                System.out.println(myContent);
              } else {
                String myToken = myStack.pop();
                string(myToken, convertAmps(myContent));
                myStack.push(myToken);
              }
          }
          // ---
          String myGroup = myMatcher.group();
          if (myGroup.startsWith("</")) { // closure
            String myToken = myGroup.substring(2).split(" |>")[0];
            String myVerif = myStack.pop();
            if (!myVerif.equals(myToken))
              throw new Exception(myVerif + " != " + myToken);
            pop(myToken);
          } else //
            if (myGroup.startsWith("<!") || myGroup.startsWith("<?")) { // comment
            comment(myGroup);
          } else //
              if (myGroup.endsWith("/>")) {
            String myToken = myGroup.substring(1).split(" |/")[0];
            // myStack.push(myToken);
            pushpop(myToken, myGroup);
            // myStack.pop();
          } else { // opening
            String myToken = myGroup.substring(1).split(" |>")[0];
            push(myToken, myGroup);
            myStack.push(myToken);
          }
        }
        myLineBuffer = new StringBuffer();
        myMatcher.appendTail(myLineBuffer);
        ++line;
      }
      if (!myStack.isEmpty()) {
        throw new Exception("xml file corrupt: " + myStack);
      }
    } finally {
      myStack = null;
    }
  }

  protected static String getValue(String myGroup, String myKey) {
    Pattern myPattern = Pattern.compile("\\s" + myKey + "\\s*=");
    Matcher myMatcher = myPattern.matcher(myGroup);
    if (myMatcher.find()) {
      int beg = myGroup.indexOf('\"', myMatcher.end()) + 1;
      int end = myGroup.indexOf('\"', beg);
      return myGroup.substring(beg, end);
    }
    return null;
  }

  protected int getLine() {
    return line;
  }

  protected String peek() {
    return myStack.peek();
  }

  protected boolean contains(String myToken) {
    return myStack.contains(myToken);
  }

  public static String convertAmps(String myString) {
    myString = myString.replaceAll("&lt;", "<");
    myString = myString.replaceAll("&gt;", ">");
    myString = myString.replaceAll("&apos;", "'");
    return myString;
  }
}
