// code by jph
package sys.gui;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.io.File;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JToggleButton;
import javax.swing.JToolBar;
import javax.swing.SwingConstants;

public final class ComponentFactory {
  public static final Dimension standard_button = new Dimension(28, 28);

  public static File getIconFile(String myString) {
    return new File("get" + File.separator + "img", myString);
  }

  /** @param myString
   * @return instance of {@link ImageIcon}, even if no image file found, i.e. never null */
  public static ImageIcon getIcon(String myString) {
    File myFile = getIconFile(myString);
    if (!myFile.exists()) {
      new RuntimeException("icon missing " + myFile.toString()).printStackTrace();
      // System.out.println();
    }
    return new ImageIcon(myFile.toString());
  }

  public static final ImageIcon ICON_TRASH = ComponentFactory.getIcon("icon.trs.png");
  public static final ImageIcon ICON_NULL = ComponentFactory.getIcon("icon.nul.png");

  public static JToolBar getJToolBar(boolean myBoolean) {
    JToolBar myJToolBar = new JToolBar();
    myJToolBar.setLayout(new FlowLayout(FlowLayout.LEFT, 3, 0));
    myJToolBar.setFloatable(false);
    if (myBoolean)
      WindowAncestor.movesBy(myJToolBar);
    return myJToolBar;
  }

  public static JToggleButton getJToggleButton(String myImageIcon, String myToolTipText) {
    JToggleButton myJToggleButton = new JToggleButton(getIcon(myImageIcon));
    myJToggleButton.setToolTipText(myToolTipText);
    return myJToggleButton;
  }

  public static JToggleButton getJToggleButton(String myImageIcon, String mySelectedImageIcon, String myToolTipText) {
    JToggleButton myJToggleButton = new JToggleButton(getIcon(myImageIcon));
    myJToggleButton.setPreferredSize(standard_button);
    myJToggleButton.setSelectedIcon(getIcon(mySelectedImageIcon));
    myJToggleButton.setBorderPainted(false);
    myJToggleButton.setToolTipText(myToolTipText);
    myJToggleButton.setContentAreaFilled(false);
    return myJToggleButton;
  }

  public static JButton getJButton(String myString, String myToolTipText) {
    JButton myJButton = new JButton(getIcon(myString));
    myJButton.setPreferredSize(new Dimension(28, 28));
    myJButton.setBorderPainted(false);
    myJButton.setOpaque(false);
    myJButton.setToolTipText(myToolTipText);
    return myJButton;
  }

  public static JLabel getJLabel(String myText, String myIcon, String myToolTipText, int width) {
    JLabel myJLabel = new JLabel(myText, SwingConstants.LEFT);
    if (myIcon != null && !myIcon.isEmpty())
      myJLabel.setIcon(getIcon(myIcon));
    myJLabel.setFont(new Font(Font.DIALOG, Font.PLAIN, 12));
    myJLabel.setToolTipText(myToolTipText);
    myJLabel.setPreferredSize(new Dimension(width, 28));
    return myJLabel;
  }
}
