// code by jph
package sys.lic;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Calendar;
import java.util.Timer;
import java.util.TimerTask;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.swing.JOptionPane;

import sys.Manager;

public class LicenseSystem {
  LicenseInterface myLicenseInterface;

  public LicenseSystem(LicenseInterface myLicenseInterface) {
    this.myLicenseInterface = myLicenseInterface;
  }

  public void start(boolean myBoolean, final Timer myTimer) {
    long period = myLicenseInterface.getPeriod();
    long first = myBoolean ? period : Manager.random.nextInt((int) period);
    if (myBoolean && !isValid())
      first = 0;
    myTimer.schedule(new TimerTask() {
      @Override
      public void run() {
        if (!isValid()) {
          new Thread(new Runnable() {
            @Override
            public void run() {
              myLicenseInterface.invalidNotification();
            }
          }).start();
          myTimer.schedule(new TimerTask() {
            @Override
            public void run() {
              myLicenseInterface.invalidAction();
            }
          }, myLicenseInterface.getDelay());
        }
      }
    }, first, period);
  }

  boolean isValid() {
    boolean myBoolean = false;
    StringBuilder myStringBuffer = new StringBuilder();
    try {
      Calendar myCalendar = InternetTime.getCalendar();
      if (myCalendar != null) {
        myStringBuffer.append("at " + License.myDateFormat.format(myCalendar.getTime()));
        for (File myFile : new File(".").listFiles(new FilenameFilter() {
          @Override
          public boolean accept(File myFile, String myString) {
            return myString.endsWith(".access");
          }
        }))
          try {
            License myLicense = License.retrieve(myLicenseInterface.getKey(), myFile);
            myBoolean |= myLicense != null && InternetTime.staticInetAddressString().startsWith(myLicense.myIPstart) && myLicense.inInterval(myCalendar);
          } catch (Exception myException) {
            myException.printStackTrace();
          }
      }
    } catch (Exception myException) {
      myException.printStackTrace();
    }
    Logger.getGlobal().log(myBoolean ? Level.INFO : Level.SEVERE, "license check " + myStringBuffer.toString() + " returned " + myBoolean + "\n");
    return myBoolean;
  }

  static Calendar createCalendar(int year, int month, int day) {
    Calendar myCalendar = Calendar.getInstance();
    myCalendar.set(Calendar.YEAR, year);
    myCalendar.set(Calendar.MONTH, month); // JANUARY is 0, etc.
    myCalendar.set(Calendar.DAY_OF_MONTH, day);
    // ---
    myCalendar.set(Calendar.HOUR_OF_DAY, 0);
    myCalendar.set(Calendar.MINUTE, 0);
    myCalendar.set(Calendar.SECOND, 0);
    myCalendar.set(Calendar.MILLISECOND, 0);
    return myCalendar;
  }

  public static void message() {
    JOptionPane.showMessageDialog(null, "Authorization invalid or expired.", "Termination required", JOptionPane.ERROR_MESSAGE);
  }
}
