// code by jph
package sys.mat;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

public class BivariateMap<P1, P2, Value> implements Serializable {
  class BiEntry {
    public final P1 myP1;
    public final P2 myP2;
    public final Value myValue;

    BiEntry(P1 myP1, P2 myP2, Value myValue) {
      this.myP1 = myP1;
      this.myP2 = myP2;
      this.myValue = myValue;
    }
  }

  private Map<P1, Map<P2, Value>> myMap = new HashMap<>();
  private final Value unput;

  /** @param unput response for empty get requests */
  public BivariateMap(Value unput) {
    this.unput = unput;
  }

  public void put(P1 myP1) {
    if (!myMap.containsKey(myP1))
      myMap.put(myP1, new HashMap<>());
  }

  public void put(P1 myP1, P2 myP2, Value myValue) {
    put(myP1);
    myMap.get(myP1).put(myP2, myValue);
  }

  public boolean containsKey(P1 myP1, P2 myP2) {
    return myMap.containsKey(myP1) && myMap.get(myP1).containsKey(myP2);
  }

  public Value get(P1 myP1, P2 myP2) {
    if (!myMap.containsKey(myP1))
      return unput;
    Map<P2, Value> row = myMap.get(myP1);
    return row.containsKey(myP2) ? row.get(myP2) : unput;
  }

  public Collection<P1> keySet() {
    return myMap.keySet();
  }

  /** @param myP1
   * @return unmodifiable map */
  public Map<P2, Value> getRowMap(P1 myP1) {
    return myMap.containsKey(myP1) ? Collections.unmodifiableMap(myMap.get(myP1)) : new HashMap<P2, Value>();
  }

  /** @param myP1
   * @param myCollection
   * @return unmodifiable map */
  public Map<P2, Value> getRowMap(P1 myP1, Collection<P2> myCollection) {
    Map<P2, Value> myReturn = new HashMap<>();
    for (P2 myP2 : myCollection)
      myReturn.put(myP2, get(myP1, myP2));
    return Collections.unmodifiableMap(myReturn);
  }

  public Map<P1, Value> getColumnMap(P2 myP2) {
    Map<P1, Value> myReturn = new HashMap<>();
    for (Entry<P1, Map<P2, Value>> myEntry : myMap.entrySet())
      if (myEntry.getValue().containsKey(myP2))
        myReturn.put(myEntry.getKey(), myEntry.getValue().get(myP2));
    return myReturn;
  }

  public Map<P1, Value> getColumnMap(P2 myP2, Collection<P1> myCollection) {
    Map<P1, Value> myReturn = new HashMap<>();
    for (P1 myP1 : myCollection)
      myReturn.put(myP1, get(myP1, myP2));
    return myReturn;
  }

  public void clear() {
    myMap.clear();
  }

  public Set<Entry<P1, Map<P2, Value>>> entrySet() {
    return myMap.entrySet();
  }

  // public Collection<BiEntry<P1, P2, Value>> entrySet() {
  // Collection<BiEntry<P1, P2, Value>> myCollection = new LinkedList<>();
  // for (Entry<P1, Map<P2, Value>> myEntry : myMap.entrySet())
  // for (Entry<P2, Value> myEntrySub : myEntry.getValue().entrySet())
  // myCollection.add(new BiEntry<P1, P2, Value>(myEntry.getKey(), myEntrySub.getKey(), myEntrySub.getValue()));
  // return myCollection;
  // }
  public BivariateMap<P2, P1, Value> transpose() {
    BivariateMap<P2, P1, Value> myBivariateMap = new BivariateMap<>(unput);
    for (Entry<P1, Map<P2, Value>> myEntry : myMap.entrySet())
      for (Entry<P2, Value> myEntrySub : myEntry.getValue().entrySet())
        myBivariateMap.put(myEntrySub.getKey(), myEntry.getKey(), myEntrySub.getValue());
    return myBivariateMap;
  }

  public void printout() {
    for (P1 myP1 : myMap.keySet()) {
      System.out.println(myP1 + " =>");
      for (Entry<P2, Value> myEntry : getRowMap(myP1).entrySet())
        System.out.println(" " + myEntry.getKey() + " -> " + myEntry.getValue());
    }
  }

  @SuppressWarnings("unchecked")
  public void incr(P1 myP1, P2 myP2) {
    Value myValue = get(myP1, myP2);
    if (myValue instanceof Integer && unput != null) {
      Integer myInteger = (Integer) myValue;
      put(myP1, myP2, (Value) (Integer) (myInteger + 1));
    }
  }
}
