// code by jph
package sys.mat;

import java.util.Map;
import java.util.NavigableMap;
import java.util.TreeMap;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

public class FrequencyMap {
  private FrequencyMap() {
  }

  /** fast
   * 
   * @param myStream
   * @return */
  public static <Type> Map<Type, Long> unsortedLong(Stream<Type> myStream) {
    return myStream.collect(Collectors.groupingBy(Function.identity(), Collectors.counting()));
  }

  /** fast
   * 
   * @param myStream
   * @return */
  public static <Type> NavigableMap<Type, Long> sortedLong(Stream<Type> myStream) {
    return new TreeMap<>(unsortedLong(myStream));
  }

  // slow
  static <Type> NavigableMap<Type, Integer> sortedInt(Stream<Type> myStream) {
    return myStream.collect(Collectors.groupingBy( //
        Function.identity(), //
        () -> new TreeMap<Type, Integer>(), //
        Collectors.reducing(0, myInt -> 1, Integer::sum)));
  }

  // slow!
  @Deprecated
  static <Type> NavigableMap<Type, Integer> sortedIntAlt(Stream<Type> myStream) {
    return myStream.collect( //
        () -> new TreeMap<>(), //
        (map, val) -> map.put(val, map.containsKey(val) ? map.get(val) + 1 : 1), //
        TreeMap::putAll);
  }
  // public static void main(String[] args) {
  // // List<Integer> myList = Arrays.asList(new Integer[] { -2, 3, 10, 2, 10, 2, 2, 2, 10, 10, 8, 2 });
  // Random r = new Random();
  // List<Integer> myList = new IntRange(10000000).stream() //
  // .map(i -> r.nextInt(10000)) //
  // .boxed() //
  // .collect(Collectors.toList());
  // PerformanceNode.insert("warm up");
  // myList.stream().collect(Collectors.toList());
  // unsortedLong(myList.stream());
  // PerformanceNode.insert("unsorted");
  // new TreeMap<>(unsortedLong(myList.stream()));
  // PerformanceNode.insert("sorted");
  // sortedInt(myList.stream());
  // PerformanceNode.insert("sorted alt");
  // sortedIntAlt(myList.stream());
  // PerformanceNode.insert();
  // }
}
