// code by jph
package sys.mat;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map.Entry;
import java.util.NavigableMap;
import java.util.TreeMap;

import sys.Manager;

public class PartitionMap {
  public final NavigableMap<Integer, Integer> myNavigableMap = new TreeMap<>();
  private int collection_size; // just for info

  public PartitionMap() {
    myNavigableMap.put(0, 0); // entry guarantees getOrdinal(0)==0
  }

  /** @param myCollection of integers to partition
   * @param size number of buckets
   * @return */
  public static PartitionMap createFrom(Collection<Integer> myCollection, int size) {
    NavigableMap<Integer, Integer> myNavigableMap = FrequencyMap.sortedInt(myCollection.stream());
    // ---
    assert(!myNavigableMap.containsKey(0));
    int count = 0;
    int thres = 0;
    int bucket = 0;
    PartitionMap myPartitionMap = new PartitionMap();
    myPartitionMap.collection_size = myCollection.size();
    for (Entry<Integer, Integer> myEntry : myNavigableMap.entrySet()) {
      count += myEntry.getValue();
      if (thres < count) {
        myPartitionMap.myNavigableMap.put(myEntry.getKey(), ++bucket);
        thres = myCollection.size() * bucket / size;
      }
    }
    return myPartitionMap;
  }

  /** @param myInt geq 0
   * @return */
  public int getOrdinal(int myInt) {
    return myNavigableMap.lowerEntry(myInt + 1).getValue();
  }

  @Override
  public String toString() {
    // the value set is 0, 1, 2, 3, ...
    // int count = 0;
    // for (int myInt : myNavigableMap.values()) {
    // if (myInt != count)
    // throw new RuntimeException("blub");
    // ++count;
    // }
    return "[" + collection_size + "] -> " + myNavigableMap.keySet().toString();
  }

  public int getCollectionSize() {
    return collection_size;
  }

  // ---
  public static void main(String[] args) {
    List<Integer> myList = new ArrayList<>();
    int upper = Manager.random.nextInt(10);
    for (int c1 = 0; c1 < upper; ++c1) {
      int num = Manager.random.nextInt(1000);
      myList.add(1 + num);
    }
    PartitionMap myPartitionMap = PartitionMap.createFrom(myList, 3);
    System.out.println(myPartitionMap);
    System.out.println(myPartitionMap.getOrdinal(400));
  }
}
