// code by jph
package sys.wav.get;

/** Base class of most {@link AudioProvider}s that manages the periodic release of audio data. */
public abstract class AudioProviderThread implements AudioProvider {
  private volatile boolean keepOpen = false;
  private volatile boolean running = false;

  @Override
  public final void openProvider(final AudioDistributor myAudioDistributor) {
    beforeOpenProvider();
    keepOpen = true;
    new Thread(new Runnable() {
      @Override
      public void run() {
        running = true;
        byte[] myByte = new byte[512];
        try {
          while (keepOpen) {
            prepareBlocking(myByte);
            myAudioDistributor.distribute512(myByte);
          }
        } catch (Exception myException) {
          // might occur when line is closed in midst of reading? ... similar to TCP/IP?
          myException.printStackTrace();
        }
        running = false;
      }
    }).start();
  }

  public abstract void prepareBlocking(byte[] myByte);

  @Override
  public final void closeProvider() {
    keepOpen = false;
    while (running)
      try {
        Thread.sleep(10);
      } catch (Exception myException) {
        myException.printStackTrace();
      }
    afterProviderClosed();
  }

  public abstract void beforeOpenProvider();

  public abstract void afterProviderClosed();
}
